#!/usr/bin/env python

import sys, math
import numpy as np
from datetime import datetime

from state import State
from point import Point
from bw import Butterworth as Bw
from sg import SG
from calibration import Calibration
from gazept import Gazept

from math import *
from OpenGL.GL import *
from OpenGL.GLUT import *
from OpenGL.GLU import *

# prevent Python from writing .pyc files
sys.dont_write_bytecode = True

def draw_circ(cx, cy, r, num_segments, type):
  theta = 2 * 3.1415926 / float(num_segments)
  # tangential factor
  tanf = tan(theta)
  # radial factor
  radf = cos(theta)
  # start x at angle 0
  x = r
  y = 0
  glBegin(type)
  i = 0

  for i in range(num_segments):
    glVertex2f(x + cx, y + cy)
    tx = -y
    ty = x
    x = x + tx * tanf
    y = y + ty * tanf
    x = x * radf
    y = y * radf

  glEnd()

def reshape(width, height):
  glViewport(0, 0, width, height)

def display():
  ww = glutGet(GLUT_WINDOW_WIDTH)
  wh = glutGet(GLUT_WINDOW_HEIGHT)

  dx = glutGet(GLUT_WINDOW_X)
  dy = glutGet(GLUT_WINDOW_Y)
  print("Getting window pos: ",dx,',',dy)

  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
  glLoadIdentity()

  glViewport(0, 0, ww, wh)
  glMatrixMode(GL_PROJECTION)
  glLoadIdentity()
  glOrtho(0.0, ww, 0.0, wh, 0.0, 1.0)
  glMatrixMode (GL_MODELVIEW)
  glLoadIdentity()

  glColor3f(1.0, 1.0, 1.0)
  draw_circ(ww/2, wh/2, 10, 100, GL_TRIANGLE_FAN)

  glutSwapBuffers()

def keyboard(bkey, x, y):

  key = bkey.decode("utf-8")

  if key == '\033':
    sys.exit( )
  if key == 'f':
    state.fullscreen = 1 - state.fullscreen
    if state.fullscreen:
      ww = glutGet(GLUT_WINDOW_WIDTH)
      wh = glutGet(GLUT_WINDOW_HEIGHT)
      x = glutGet(GLUT_WINDOW_X)
      y = glutGet(GLUT_WINDOW_Y)
      print("window pos =", x, y)
      if sys.platform == 'win32':
        # subtract width of window decorations (Windows)
        x -= 8
        y -= 30
      state.set_window_width(ww)
      state.set_window_height(wh)
      state.set_window_pos(x,y)
      glutFullScreen()
    else:
      ww = state.get_window_width()
      wh = state.get_window_height()
      dx = state.get_window_pos_x()
      dy = state.get_window_pos_y()
      glutReshapeWindow(ww,wh)
      glutPositionWindow(dx,dy)

def main():
  global state
  state = State()

  state.set_window_width(512)
  state.set_window_height(512)
# state.set_window_width(1680)
# state.set_window_height(1050)
  state.set_screen(17)
  state.set_viewing_dist(19.68)
  state.set_mouse_coord_x(.5)
  state.set_mouse_coord_y(.5)
  state.set_dfwidth(5)
  state.set_dfdegree(2)
  state.set_dfo(1)
  state.set_threshold(20)

  glutInit(sys.argv)
  glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA | GLUT_ALPHA | GLUT_DEPTH)

  ww = state.get_window_width()
  wh = state.get_window_height()
  sw = glutGet(GLUT_SCREEN_WIDTH)
  sh = glutGet(GLUT_SCREEN_HEIGHT)
  dx = (sw - ww)//2
  dy = (sh - wh)//2
  print("Setting window pos: ",dx,',',dy)
  state.set_window_pos(dx,dy)

  mmw = glutGet(GLUT_SCREEN_WIDTH_MM)
  mmh = glutGet(GLUT_SCREEN_HEIGHT_MM)
  print("screen size (mm): ",mmw,',',mmh)
  inw = glutGet(GLUT_SCREEN_WIDTH_MM) * 0.0393701
  inh = glutGet(GLUT_SCREEN_HEIGHT_MM) * 0.0393701
  print("screen size (in): ",inw,',',mmh)
  inr = math.sqrt(float(inw)*float(inw) + float(inh)*float(inh))
  print("screen diagonal (in): ",inr)
  state.set_screen(inr)

  window = glutCreateWindow('Gazepoint client demo: press Esc to exit')
  glutReshapeWindow(ww,wh)
  glutPositionWindow(dx,dy)
# glutFullScreen()
  glutReshapeFunc(reshape)
  glutDisplayFunc(display)
  glutKeyboardFunc(keyboard)

  glutSetWindow(window)

  glClearColor(0.0,0.0,0.0,0.0)

  glutMainLoop()

#################################
if __name__ == '__main__':
  main()
