import socket, sys, os, copy, math, time, datetime, unicodedata
from threading import Event, Lock, Thread
from multiprocessing import Queue
from OpenGL.GLUT import *
from OpenGL.GLU import *
from OpenGL.GL import *
import numpy as np
from xml.etree import ElementTree as ET
import lxml.etree

# prevent Python from writing .pyc files
sys.dont_write_bytecode = True

class Gazept:
  # laptop in lab ip: 130.127.48.129   10.211.55.3
# def __init__(self, server='130.127.48.128', port=4242):
# def __init__(self, server='129.132.127.147', port=4242):
  def __init__(self, server='127.0.0.1', port=4242, debug=True):

    self.server = server
    self.port = port

    self._debug = debug
    if self._debug:
      dt = time.strftime("%Y-%m-%d_%H-%M-%S")
      self._debuglog = open('debug_%s.txt' % (dt), 'w')
      self._debuglog.write("DEBUG LOG %s\n" % (dt))

    try:
      # Connect to the server
      self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
      # non-blocking socket...more difficult to work with?
#     self.sock.setblocking(0)
      # blocking socket
      self.sock.settimeout(1.0)
      self.sock.connect((server, port))
      self._debug_print('connected to %s' % (server))
      # connected, calibrating, receiving, shutdown
      self.state = 'connected'
      self.calx = self.caly = 0.5
      self.p2x = self.p2y = 0.5
      self.lpcx = self.lpcy = self.lps = 0.0
      self.rpcx = self.rpcy = self.rps = 0.0
      self.bpogx = self.bpogy = 0.0

      self._unfinished = ''

      self._queue = Queue()

      # Start a thread for receiving
      self.receiving_thread = Thread(target = self.communication_loop)
      self.receiving_thread.start()
    except:
      self._debug_print('couldn\'t connect to %s:%s' % (server,port))
      self._debug_print('setting state to mouse coords')
      raise
#     sys.exit()

  # private method
  def send(self, msg):
#   self.sock.send(str.encode(msg + '\r\n','ascii'))
    self.sock.send(str.encode(msg + '\r\n'))

  # send points to eye tracker and set timeout and delay
  def calibrate_addpoint(self, x, y):
    self.send(self.toxml('SET', ID="CALIBRATE_ADDPOINT", \
                                X=str(x),Y=str(y)))

  def screen_size(self,x,y,w,h):
    self.send(self.toxml('SET', ID="SCREEN_SIZE", \
                                X=str(x),Y=str(y),WIDTH=str(w),HEIGHT=str(h)))

  def tracker_display(self,state):
    self.send(self.toxml('SET', ID="TRACKER_DISPLAY", STATE=str(state)))

  def calibrate_show(self,state):
    self.send(self.toxml('SET', ID="CALIBRATE_SHOW", STATE=str(state)))

  def calibrate_clear(self):
    self.send(self.toxml('SET', ID="CALIBRATE_CLEAR"))

  def calibrate_timeout(self,val):
    self.send(self.toxml('SET', ID="CALIBRATE_TIMEOUT", VALUE=str(val)))

  def calibrate_delay(self,val):
    self.send(self.toxml('SET', ID="CALIBRATE_DELAY", VALUE=str(val)))

  def start_calibration(self):
    self.send(self.toxml('SET', ID="CALIBRATE_START", STATE="1"))
    self.state = 'calibrating'

  def stop_calibration(self):
    self.send(self.toxml('SET', ID="CALIBRATE_START", STATE="0"))
    self.send(self.toxml('GET', ID="CALIBRATE_RESULT_SUMMARY"))
    self.start_receiving()

  def send_counter(self,state):
    self.send(self.toxml('SET', ID="ENABLE_SEND_COUNTER", STATE=str(state)))

  def send_cursor(self,state):
    self.send(self.toxml('SET', ID="ENABLE_SEND_CURSOR", STATE=str(state)))

  def send_eye_left(self,state):
    self.send(self.toxml('SET', ID="ENABLE_SEND_EYE_LEFT", STATE=str(state)))

  def send_eye_right(self,state):
    self.send(self.toxml('SET', ID="ENABLE_SEND_EYE_RIGHT", STATE=str(state)))

  def send_pog_fix(self,state):
    self.send(self.toxml('SET', ID="ENABLE_SEND_POG_FIX", STATE=str(state)))

  def send_pog_left(self,state):
    self.send(self.toxml('SET', ID="ENABLE_SEND_POG_LEFT", STATE=str(state)))

  def send_pog_right(self,state):
    self.send(self.toxml('SET', ID="ENABLE_SEND_POG_RIGHT", STATE=str(state)))

  def send_pog_best(self,state):
    self.send(self.toxml('SET', ID="ENABLE_SEND_POG_BEST", STATE=str(state)))

  def send_pupil_left(self,state):
    self.send(self.toxml('SET', ID="ENABLE_SEND_PUPIL_LEFT", STATE=str(state)))

  def send_pupil_right(self,state):
    self.send(self.toxml('SET', ID="ENABLE_SEND_PUPIL_RIGHT", STATE=str(state)))

  def send_time(self,state):
    self.send(self.toxml('SET', ID="ENABLE_SEND_TIME", STATE=str(state)))

  def send_time_tick(self,state):
    self.send(self.toxml('SET', ID="ENABLE_SEND_TIME_TICK", STATE=str(state)))

  def send_user_data(self,state):
    self.send(self.toxml('SET', ID="ENABLE_SEND_USER_DATA", STATE=str(state)))

  def send_data(self,state):
    self.send(self.toxml('SET', ID="ENABLE_SEND_DATA", STATE=str(state)))

  def user_data(self,data):
    self.send(self.toxml('SET', ID="USER_DATA", VALUE=str(data)))

  def start_receiving(self):
    self.send_data(1)
    self.state = 'receiving'

  def stop_receiving(self):
    self.send_data(0)
    self.state = 'connected'

  def _debug_print(self, msg):
    if self._debug:
      self._debuglog.write("%s: %s\n" % \
        (datetime.datetime.now().strftime("%H:%M:%S.%f"),msg))
      self._debuglog.flush()

  def print_err_message(self, element):
    msg = element.tag + ' '
    for key, val in element.attrib.items():
      msg += key + ':' + val + ' '
    msg += '\n'
    self._debug_print(msg)

  # private method called by communication loop
  def handle_message(self, bmsg):
    # neither of these appear to be needed with Python3
#   msg = unicodedata.normalize('NFKD', bmsg).encode('ascii','ignore')
#   msg = bmsg.decode('utf-8')
    msg = bmsg
    self._debug_print("parsing: %s" % (msg))

    # xml.etree: parsing
    # the server can send data with four different tags: ACK, NACK, CAL, REC
#   element = ET.fromstring(msg)
    element = lxml.etree.fromstring(msg)
#   element = lxml.etree.fromstring(msg,parser=lxml.etree.XMLParser(recover=True))
    tag = element.tag
    
    # 'ACK' in response to 'GET' and 'SET': 
    if 'ACK' in tag:
#     if self.state=='connected':
      if True:
        print(tag, ' ', end='')
        for key, val in element.attrib.items():
          print(key, ':', val, end=' ')
        print('\n', end='')
        for key, val in element.attrib.items():
          if key == 'ID':
            print(val,"with",sep=' ',end=' ')
          else:
            print(key,"=",val,sep=' ',end=' ')
        print("ACKNOWLEDGED",flush=True)
      else:
        self._debug_print("Warning: got 'ACK' when %s" % (self.state))
        self.print_err_message(element)

    # 'NACK' in response to 'GET' and 'SET': 
    elif 'NACK' in tag:
#     if self.state=='connected':
      if True:
        print(tag, ' ', end='')
        for key, val in element.attrib.items():
          print(key, ':', val, end=' ')
        print('\n', end='')
        for key, val in element.attrib.items():
          if key == 'ID':
            print(val,"with",sep=' ',end=' ')
          else:
            print(key,"=",val,sep=' ',end=' ')
        print("NOT ACKNOWLEDGED",flush=True)
      else:
        self._debug_print("Warning: got 'NACK' when %s" % (self.state))
        self.print_err_message(element)

    # 'CAL' during calibration
    elif 'CAL' in tag:
#     if self.state=='calibrating':
      if True:
        for key, val in element.attrib.items():
          if key == 'CALX':
              self.calx = float(val)
          if key == 'CALY':
              self.caly = float(val)
          if key == 'AVE_ERROR':
            self.error = float(val)
            print("Average Error: ", self.error)
          if key == 'VALID_POINTS':
            self.validity = float(val)
            print("Validity of points: ", self.validity)
      else:
        self._debug_print("Warning: got 'CAL' tag when %s" % (self.state))
        self.print_err_message(element)

    # 'REC' when receiving data
    elif 'REC' in tag:
      if self.state=='receiving':
        for key, val in element.attrib.items():
          if key == 'BPOGX':
            self.bpogx = float(val)
          if key == 'BPOGY':
            self.bpogy = float(val)
          if key == 'FPOGX':
            self.fpogx = float(val)
          if key == 'FPOGY':
            self.fpogy = float(val)
          if key == 'LPOGX':
            self.lpogx = float(val)
          if key == 'LPOGY':
            self.lpogy = float(val)
          if key == 'RPOGX':
            self.rpogx = float(val)
          if key == 'RPOGY':
            self.rpogy = float(val)
   
          if key == 'LPCX':
            self.lpcx = float(val)
          if key == 'LPCY':
            self.lpcy = float(val)
          if key == 'LPS':
            self.lps = float(val)
  
          if key == 'RPCX':
            self.rpcx = float(val)
          if key == 'RPCY':
            self.rpcy = float(val)
          if key == 'RPS':
            self.rps = float(val)
        # put data on queue
        self._queue.put((self.bpogx,self.bpogy))
      else:
        self._debug_print("Warning: got 'REC' tag when %s" % (self.state))
        self.print_err_message(element)

    # error: unknown tag
    else:
      self._debug_print("Warning: unknown tag: %s" % (tag))
      self.print_err_message(element)

  # private method. called by __init__
  # based on Edwin Dalmaijer's PyOpenGaze (Version 1 27-Apr-2016)
  def communication_loop(self):

    while self.state != 'shutdown':

      # receive a message
      timeout = False
      try:
#       rxdat = self.sock.recv(4096)
        rxdat = self.sock.recv(1024)
      except OSError as err:
        self._debug_print("OS error: {0}".format(err))
        continue
      except socket.timeout:
        timeout = True

      # if nothing came in, skip processing, return to top of while loop
      if timeout:
        self._debug_print("recv() timeout")
        continue

      # note: this was str.decode(rxdat) which didn't do anything
      instring = bytes.decode(rxdat)
      self._debug_print("raw instring=%s" % (instring))

      # split messages (they are separated by '\r\n')
      messages = instring.split('\r\n')

      # remove any '' from messages list
      messages = [s for s in messages if s]

      self._debug_print("unfinished=%s" % (self._unfinished))
      self._debug_print("messages=%s" % (messages))

      # check if there is currently an unfinished message
      if self._unfinished:
        # combine currently unfinished message and most recent incoming msg
        messages[0] = copy.copy(self._unfinished) + messages[0]
        self._debug_print("messages=%s" % (messages))
        self._unfinished = ''
        self._debug_print("unfinished=%s" % (self._unfinished))

      # check if the last message was complete
      if not messages[-1][-2:] == '/>':
        self._debug_print("popping %s" % (messages[-1]))
        self._unfinished = messages.pop(-1)
        self._debug_print("unfinished=%s" % (self._unfinished))

      for message in messages:
        self.handle_message(message)

      # it's possible we're shutting down and we tripped the
      # socket time out. See if that was the case: state will be shutdown.
      if self.state == 'shutdown':
        return

  # private method. called by any function composing XML strings
  def toxml(self,tag,**kwargs):
    element = ET.Element(tag,kwargs)
    return(ET.tostring(element).decode('utf-8'))

  # public method called by keyboard when escape is pressed
  def shutdown(self):
    self.state = 'shutdown'

    # shuedown socket
    self._debug_print('closing socket: ...')
    self.sock.close()
    self._debug_print('closing socket: done')

    # shuedown quuee
    self._debug_print('closing queue: ...')
    self._queue.close()
    self._queue.join_thread()
    self._debug_print('closing queue: done')

  def get_state(self):
    return self.state

  def get_gaze_coord(self):
    # this function is called by the UI/display thread that runs too frequently 
    # if we don't block we'll continually feed the display thread the same
    # value until a new one comes in
#   x, y = self.bpogx, self.bpogy

    # therefore we should block until we have new data; this is somewhat
    # dangerous in that we don't have a timeout, hence the display thread
    # can get stuck here, if, for example, the eye tracking server fails
    (x,y) = self._queue.get(block=True,timeout=None)

    # we should probably rewrite the above to be of this form:
#   while not self._shutdown_event.is_set():
#     try:
#       (x,y) = self._queue.get(block=True,timeout=1.0)
#     except self._queue.Empty:
#       continue

    return x,y

  def get_gaze_coord_x(self):
    # runs too fast
    return self.bpogx

  def get_gaze_coord_y(self):
    # runs too fast
    return self.bpogy
