#!/usr/bin/env python3
# Monitor.py

# system includes
import sys
import os
import math
import numpy as np
from scipy import spatial

class Monitor:

  #        n: number of target points
  #        S: the actual (known) locations of the n target points
  def __init__(self,wx=0,wy=0,ww=1920,wh=1200,sw=1920,sh=1200,screen=22,dist=21.65,n=5):
    self.n = n
    self.S = np.matrix(np.zeros((self.n,2),dtype=float))

    # window position and resolution, same as w, h below (pixels)
    self.pos = [wx, wy]
    self.res = [ww, wh]
    self.ww = float(ww)
    self.wh = float(wh)

    # fullscreen width and height (unknown)
    self.sw = float(sw)
    self.sh = float(sh)

    # diagonal screen dimension (inches)
    self.screen = float(screen)

    # screen width, height
    self.r = math.sqrt(self.sw*self.sw + self.sh*self.sh)
    self.dpi = self.r/self.screen

    # viewing distance (inches)
    self.D = float(dist)

    # field of view
#   fov = 2*math.degrees(math.atan(math.radians(screen/(2*self.D))))
    fov = 2*math.degrees(math.atan2(screen,2*self.D))
    fovx = 2*math.degrees(math.atan2(float(ww)/self.dpi,2*self.D))
    fovy = 2*math.degrees(math.atan2(float(wh)/self.dpi,2*self.D))


#   print "screen/window subtends %f (%f x %f) degrees" % \
#                   (float(fov),float(fovx),float(fovy))
#   print "screen aspect = %f" % float(float(sw)/float(sh))
#   print "self.dpi = %f" % self.dpi

    # get pixels for degree visual angle (10 deg offser for targets)
    degs = 10.0
#   dpix = 2*self.D*math.tan(math.radians(degs/2.0)) * self.dpi
    dpix = self.deg2pix(degs)
    dx = dpix/float(ww)
    dy = dpix/float(wh)

    # in (0,0) top-left normalized coordinates

    self.S[0,0] = 0.5		# center
    self.S[0,1] = 0.5

    self.S[1,0] = 0.5 + dx	# bottom_right
    self.S[1,1] = 0.5 + dy

    self.S[2,0] = 0.5 - dx	# bottom_left
    self.S[2,1] = 0.5 + dy

    self.S[3,0] = 0.5 - dx	# top_left
    self.S[3,1] = 0.5 - dy
 
    self.S[4,0] = 0.5 + dx	# top_right
    self.S[4,1] = 0.5 - dy

    self.ndict = {'center':0,\
                  'bottom_right':1,\
                  'bottom_left':2,\
                  'top_left':3,\
                  'top_right':4}

  def deg2pix(self,degs):
    dpix = 2*self.D*math.tan(math.radians(degs/2.0)) * self.dpi
    return dpix

  def pix2deg(self,dpix):
    r = math.sqrt(self.sw*self.sw + self.sh*self.sh)
    # pixel distances are not coming in are normalized
#   dpix = dpix * r
    degs = 2*math.degrees(math.atan2(dpix/self.dpi,2*self.D))
    return degs

  def map2ctr(self,x,y):
    # map [0,1] -> [-1,1]
    x = x * 2.0 - 1.0
    y = y * 2.0 - 1.0
    return x,y

  def set_fullscreen(self,w,h):
    self.sw = w
    self.sh = h

  def set_fullscreen_width(self,w):
    self.sw = w

  def set_fullscreen_height(self,h):
    self.sh = h

  def get_fullscreen_width(self):
    return self.sw

  def get_fullscreen_height(self):
    return self.sh

  def get_window_width(self):
    return self.res[0]

  def get_window_height(self):
    return self.res[1]

  def get_window_pos_x(self):
    return self.pos[0]

  def get_window_pos_y(self):
    return self.pos[1]

  def set_window_pos(self,x,y):
    self.pos[0] = x
    self.pos[1] = y

  def set_window_width(self,w):
    self.res[0] = w
    self.ww = float(w)

  def set_window_height(self,h):
    self.res[1] = h
    self.wh = float(h)

  def set_screen(self,s):
    self.screen = s

  def get_screen(self):
    return self.screen

  def set_viewing_dist(self,d):
    self.D = d

  def get_viewing_dist(self):
    return self.D
