# RunningStats.py
# This class represents a running mean, stdev, and variance
# from: https://stackoverflow.com/questions/1174984/how-to-efficiently-calculate-a-running-standard-deviation

# system includes
import sys
import math

# prevent Python from writing .pyc files
sys.dont_write_bytecode = True

class RunningStats:

  def __init__(self):
    self.n = 0
    self.old_m = 0
    self.new_m = 0
    self.old_s = 0
    self.new_s = 0

  def clear(self):
    self.n = 0

  def push(self, x):
    self.n += 1

    if self.n == 1:
      self.old_m = self.new_m = x
      self.old_s = 0
    else:
      self.new_m = self.old_m + (x - self.old_m) / self.n
      self.new_s = self.old_s + (x - self.old_m) * (x - self.new_m)

      self.old_m = self.new_m
      self.old_s = self.new_s

  def avg(self):
    return self.new_m if self.n else 0.0

  def var(self):
    return self.new_s / (self.n - 1) if self.n > 1 else 0.0

  def sdv(self):
    return math.sqrt(self.var())
